import tkinter as tk
import tkinter.font as font
import pandas as pd
from tkinter import *
from tkinter.ttk import *
from PIL import Image
from PIL import ImageTk
from datetime import datetime
import matplotlib.pyplot as plt
import seaborn as sns
from matplotlib.pyplot import figure
import networkx as nx


def set_main_window(window_name, width=1200, height=700):
    window = Tk()
    window.title(f"{window_name}")
    window.geometry(f"{width}x{height}")
    return window


def set_sub_window(window_name, width=1200, height=700):
    sub_window = Toplevel(main_window)
    sub_window.title(f"{window_name}")
    sub_window.geometry(f"{width}x{height}")
    sub_window.withdraw()
    return sub_window


def make_image(path, width=50, height=50):
    image = Image.open(f"{path}")
    image = image.resize((width, height))
    return ImageTk.PhotoImage(image)


def add_button(window, text, x, y, logo, command=None):
    if text != "":
        Button(window, text=text, image=logo, compound=LEFT, command=command).place(x=x, y=y)
    else:
        Button(window, text=text, image=logo, command=command).place(x=x, y=y)


def add_entry(window, x, y, width=13):
    entry = Entry(window, width=width)
    entry.place(x=x, y=y)
    return entry


def add_label(window, text, x, y, size=30):
    my_font = font.Font(family='Helvetica', size=size)
    label = tk.Label(window, text=text)
    label.place(x=x, y=y)
    label['font'] = my_font


def show_table(window, data, table_name):
    columns = list(data.columns)

    my_font = font.Font(family='Helvetica', size=40, weight="bold")
    label = tk.Label(window, text=table_name)
    label.grid(row=20)
    label['font'] = my_font

    header_font = font.Font(family='Helvetica', size=30,weight="bold")
    for i, col_name in enumerate(columns, start=0):
        label = tk.Label(window, text=col_name)
        label.grid(row=23, column=i, padx=30)
        label["font"] = header_font

    for i, row in enumerate(data.values[len(data) - 18:], start=27):
        for col in range(0, len(columns)):
            tk.Label(window, text=row[col]).grid(row=i, column=col)


def close():
    main_window.destroy()


def open_table_close_main():
    main_window.withdraw()
    table_window.deiconify()


def open_main_close_table():
    table_window.withdraw()
    main_window.deiconify()


def open_transaction_close_table():
    table_window.withdraw()
    transaction_table_window.deiconify()


def open_account_close_table():
    table_window.withdraw()
    account_table_window.deiconify()


def open_table_close_transaction():
    transaction_table_window.withdraw()
    table_window.deiconify()


def open_table_close_account():
    account_table_window.withdraw()
    table_window.deiconify()


def open_transfer_close_main():
    main_window.withdraw()
    transaction_table_window.withdraw()
    account_table_window.withdraw()
    table_window.withdraw()
    transfer_window.deiconify()


def open_main_close_transfer():
    transfer_window.withdraw()
    main_window.deiconify()


def open_graph_close_main():
    main_window.withdraw()
    graph_window.deiconify()


def open_main_close_graph():
    graph_window.withdraw()
    main_window.deiconify()


def open_graph_close_show_graph():
    show_graph_window.destroy()
    graph_window.deiconify()


def transfer_checking(sender_account, receiver_account, amount):
    account_list = list(account_df["AccountNumber"])
    if sender_account in account_list and receiver_account in account_list:
        if sender_account != receiver_account:
            if amount > 0:  # Check sender balance and update balance for both sender and receiver and return button
                return True
            else:
                print("Amount must be greater than 0!")
        else:
            print("Sender and receiver can't be the same account!")
    else:
        print("Sender or receiver account is invalid")


def transfer():
    global transaction_df, transaction_table_window, account_table_window
    sender_account, receiver_account = int(sender_entry.get()), int(receiver_entry.get())
    amount = float(amount_entry.get())
    if transfer_checking(sender_account, receiver_account, amount):
        transaction = {"Timestamp": datetime.now(),
                       "Sender": sender_account,
                       "Receiver": receiver_account,
                       "Method": "Online",
                       "Amount": amount}
        transaction_df = transaction_df.append(transaction, ignore_index=True)
        transaction_df.to_csv("data/transactions.csv", index=False)
        transaction_table_window.withdraw()
        account_table_window.withdraw()
        transaction_table_window = set_sub_window("Transaction Table")
        account_table_window = set_sub_window("Account Table")
        add_button(transaction_table_window, "", 900, 530, return_logo, open_table_close_transaction)
        add_button(account_table_window, "", 900, 530, return_logo, open_table_close_account)
        transaction_table_window.withdraw()
        account_table_window.withdraw()
        show_table(transaction_table_window, transaction_df, "Transaction")
    else:
        print("Out")


def add_graph_to_show_graph_window(number=None):
    plt.clf()
    global img, show_graph_window
    show_graph_window = set_sub_window("Show Graph")
    add_button(show_graph_window, "", 900, 530, return_logo, open_graph_close_show_graph)
    graph_window.withdraw()
    img = ImageTk.PhotoImage(Image.open(f"graphs/graph.png"))
    panel = Label(show_graph_window, image=img)
    if number:
        panel.place(x=60,y=50)
    else:
        panel.place(x=150, y=50)
    show_graph_window.deiconify()


def generate_graph_1():
    figure(figsize=(6, 5),dpi=100)
    plt.hist(account_df["Balance"])
    plt.xlabel("Balance")
    plt.ylabel("Count")
    plt.title("Histogram of people's balance in the bank account")
    plt.savefig('graphs/graph.png')
    add_graph_to_show_graph_window()


def generate_graph_2():
    figure(figsize=(6, 5),dpi=100)
    sns.countplot(x="Bank", data=account_df, order=account_df['Bank'].value_counts().index)
    plt.xlabel("Bank")
    plt.ylabel("Count")
    plt.title("Bar chart for the number of people in each bank")
    plt.savefig('graphs/graph.png')
    add_graph_to_show_graph_window()


def generate_graph_3():
    figure(figsize=(13, 5), dpi=70)
    transaction = transaction_df.copy()
    transaction["DayOfWeek"] = [date.day_name() for date in transaction["Timestamp"]]
    plt.plot(transaction.groupby(by="DayOfWeek")[["Amount"]].mean(), label="Mean")
    plt.xlabel("Day Of Week")
    plt.ylabel("Value")
    plt.title("Statistical measurement of the transaction for each day of week")
    plt.legend(loc='center left', bbox_to_anchor=(1.0, 0.5))
    plt.savefig('graphs/graph.png')
    add_graph_to_show_graph_window(3)


def generate_graph_4():
    transaction = transaction_df.copy()
    transaction["Month"] = [date.month_name() for date in transaction["Timestamp"]]
    figure(figsize=(12, 6), dpi=70)
    plt.plot(transaction.groupby(by="Month")[["Amount"]].mean(), label="Mean")
    plt.xlabel("Month")
    plt.ylabel("Value")
    plt.title("Statistical measurement of the transaction for each month")
    plt.legend(loc='center left', bbox_to_anchor=(1.0, 0.5))
    plt.savefig('graphs/graph.png')
    add_graph_to_show_graph_window(4)


def generate_graph_5():
    transaction = transaction_df.copy()
    temp_df = transaction[transaction.Method == 'Mobile']
    column_list = ['Sender', 'Receiver']
    G = nx.Graph()

    for row_index in temp_df.index:
        # Add node using sender and receiver
        for col_name in column_list:
            transact = temp_df.loc[row_index, col_name]
            G.add_node(transact)

        # Add edge by sender and receiver
        for i in range(0, len(column_list) - 1):
            sender = temp_df.loc[row_index, column_list[i]]
            for j in range(i + 1, len(column_list)):
                receiver = temp_df.loc[row_index, column_list[j]]
                G.add_edge(sender, receiver)

    pos = nx.spring_layout(G)
    nx.draw(G, pos, with_labels=True)
    plt.title("Network graph between sender and receiver")
    plt.savefig('graphs/graph.png')
    add_graph_to_show_graph_window(5)


# Data
transaction_df = pd.read_csv("data/transactions.csv")
transaction_df["Timestamp"] = pd.to_datetime(transaction_df["Timestamp"])
account_df = pd.read_csv("data/accounts.csv")

# Windows
main_window = set_main_window("Main Window")
table_window = set_sub_window("Table Window")
transaction_table_window = set_sub_window("Transaction Table")
account_table_window = set_sub_window("Account Table")
transfer_window = set_sub_window("Transfer Window")
graph_window = set_sub_window("Graph Results")
show_graph_window = set_sub_window("Show Graph")

img = ImageTk.PhotoImage(Image.open(f"graphs/graph.png"))

add_label(main_window, "Main Menu", 430, 100, 50)
add_label(table_window, "Select Table", 410, 100, 50)

# Add table to windows
show_table(transaction_table_window, transaction_df, "Transaction")
show_table(account_table_window, account_df, "Account")

# Logos and buttons
table_logo = make_image("images/table.png")
add_button(main_window, "Show Tables", 200, 300, table_logo, open_table_close_main)
transfer_logo = make_image("images/transfer.png")
add_button(main_window, "Transfer", 525, 300, transfer_logo, open_transfer_close_main)

graph_logo = make_image("images/chart.png")
add_button(main_window, "Graphs", 900, 300, graph_logo, open_graph_close_main)

transaction_logo = make_image("images/transaction.png")
add_button(table_window, "Transaction Table", 300, 300, transaction_logo, open_transaction_close_table)

account_logo = make_image("images/account.png")
add_button(table_window, "Account Table", 700, 300, account_logo, open_account_close_table)

exit_logo = make_image("images/exit.png")

return_logo = make_image("images/back.png")
add_button(table_window, "", 550, 600, return_logo, open_main_close_table)
add_button(transaction_table_window, "", 550, 600, return_logo, open_table_close_transaction)
add_button(account_table_window, "", 550, 600, return_logo, open_table_close_account)
add_button(transfer_window, "", 550, 600, return_logo, open_main_close_transfer)
add_button(graph_window, "", 550, 600, return_logo, open_main_close_graph)
add_button(show_graph_window, "", 550, 600, return_logo, open_graph_close_show_graph)

# Entries and Button
sender_entry = add_entry(transfer_window, 300, 160)
receiver_entry = add_entry(transfer_window, 750, 160)
amount_entry = add_entry(transfer_window, 545, 320)
add_label(transfer_window, "Sender", 280, 100)
add_label(transfer_window, "Receiver", 710, 100)
add_label(transfer_window, "Amount", 520, 260)
add_button(transfer_window, "", 550, 450, transfer_logo, transfer)

# add button graph window
add_button(graph_window, "Histogram of people's balance in the bank account", 420, 100, graph_logo, generate_graph_1)
add_button(graph_window, "Bar chart for the number of people in each bank", 430, 200, graph_logo, generate_graph_2)
add_button(graph_window, "Statistical measurement of the transaction for each day of week", 400, 300, graph_logo,
           generate_graph_3)
add_button(graph_window, "Statistical measurement of the transaction for each month", 410, 400, graph_logo,
           generate_graph_4)
add_button(graph_window, "Network graph between sender and receiver", 450, 500, graph_logo,
           generate_graph_5)

add_button(main_window, "Exit", 525, 600, exit_logo, close)
mainloop()
